<?php

namespace Drupal\jsonapi_advanced_permissions\Permissions;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Routing\RouteProviderInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides dynamic permissions for JSON:API collections.
 */
class JsonApiAdvancedPermissions implements ContainerInjectionInterface {

  use StringTranslationTrait;

  /**
   * The route provider service.
   *
   * @var \Drupal\Core\Router\RouteProviderInterface
   */
  protected $routeProvider;

  /**
   * The config service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a JsonApiAdvancedPermissions object.
   *
   * @param \Drupal\Core\Router\RouteProviderInterface $route_provider
   *   The route provider service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   */
  public function __construct(
    RouteProviderInterface $route_provider,
    ConfigFactoryInterface $config_factory,
  ) {
    $this->routeProvider = $route_provider;
    $this->configFactory = $config_factory;
  }

  /**
   * Factory method to create an instance of JsonApiAdvancedPermissions.
   *
   * @return \Drupal\jsonapi_advanced_permissions\Permissions\JsonApiAdvancedPermissions
   *   A JsonApiAdvancedPermissions instance.
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('router.route_provider'),
      $container->get('config.factory')
    );
  }

  /**
   * Returns dynamic permissions for JSON:API collections.
   *
   * @return array
   *   An array of permissions.
   */
  public function permissions() {
    $permissions = [];
    $routes = $this->routeProvider->getAllRoutes();
    foreach ($routes as $route_name => $route) {
      $defaults = $route->getDefaults();
      if (!empty($defaults['_is_jsonapi']) && !empty($defaults['resource_type'])) {
        $resourceType = $defaults['resource_type'];
        $resourceTypeName = str_replace('--', ' ', $resourceType);
        $resourceTypeName = str_replace('_', ' ', $resourceTypeName);
        $configParts = $this->configFactory->get('jsonapi_advanced_permissions.settings')->get('permissions') ?? [];
        foreach ($configParts as $configPart) {
          if ($route_name == 'jsonapi.' . $defaults['resource_type'] . '.' . $configPart) {
            $methods = $route->getMethods();
            $configPartName = explode('.', $configPart)[0];
            $configPartName = str_replace('.', ' ', $configPartName);
            $configName = ucwords($resourceTypeName . ' ' . $configPartName);
            $resourceTypePermission = str_replace('.', '--', ($resourceType . '--' . $configPart));
            $resourceTypePermission = str_replace('_', '-', $resourceTypePermission);

            foreach ($methods as $method) {
              $permissionName = str_replace('jsonapi.', '', $route_name);
              $permissionName = str_replace('--', '__', $permissionName);
              $permissionName = str_replace('.', ' ', $permissionName);
              $methodName = strtolower($method);
              $permissions["jsonapi_advanced_permissions $methodName on $permissionName"] = [
                'title' => $this->t('Access @method on @collection',
                  [
                    '@method' => $method,
                    '@collection' => $configName,
                  ]),
              ];
            }
          }
        }
      }
    }
    return $permissions;
  }

}
